@@ -1163,6 +1163,9 @@ create_panel_from_desktop (void)
 					marked_count++;
 					fe->f.marked = TRUE;
 
+					if (dii == last_selected_icon)
+						selected_index = count;
+
 					if (S_ISDIR (fe->buf.st_mode)) {
 						dir_marked_count++;
 						if (fe->f.dir_size_computed)
@@ -1285,11 +1288,7 @@ desktop_icon_info_open (DesktopIconInfo *dii)
 	} else {
 		WPanel *panel;
 		panel = push_desktop_panel_hack ();
-		/* we need to special case the new dir.  Otherwise we'd try to cd */
-		if (S_ISDIR (fe->buf.st_mode) || link_isdir (fe))
-			new_panel_at (filename);
-		else
-			do_enter_on_file_entry (fe);
+		do_enter_on_file_entry (fe);
 		layout_panel_gone (panel);
 		free_panel_from_desktop (panel);
 	}
@@ -2380,6 +2379,7 @@ handle_rescan_desktop (GtkWidget *widget, gpointer data)
 {
 	desktop_reload_icons (FALSE, 0, 0);
 }
+
 static void
 set_background_image (GtkWidget *widget, gpointer data)
 {
@@ -2407,6 +2407,7 @@ static GnomeUIInfo gnome_panel_new_menu [] = {
 	GNOMEUIINFO_ITEM_NONE(N_("_Terminal"), N_("Launch a new terminal in the current directory"), gnome_open_terminal),
 	/* If this ever changes, make sure you update create_new_menu accordingly. */
 	GNOMEUIINFO_ITEM_NONE( N_("_Directory..."), N_("Creates a new directory"), gnome_mkdir_cmd ),
+	GNOMEUIINFO_ITEM_NONE( N_("URL L_ink..."),  N_("Creates a new URL link"), gnome_new_link ),
 	GNOMEUIINFO_ITEM_NONE( N_("_Launcher..."), N_("Creates a new launcher"), gnome_new_launcher ),
 	GNOMEUIINFO_END
 };
@@ -2650,6 +2651,22 @@ click_proxy_button_press (GtkWidget *widget, GdkEventButton *event, gpointer dat
 	return FALSE;
 }
 
+/* Terminates rubberbanding when the button is released.  This is shared by the
+ * button_release handler and the motion_notify handler.
+ */
+static void
+perform_release (guint32 time)
+{
+	draw_rubberband (click_current_x, click_current_y);
+	gdk_pointer_ungrab (time);
+	click_dragging = FALSE;
+
+	update_drag_selection (click_current_x, click_current_y);
+
+	XUngrabServer (GDK_DISPLAY ());
+	gdk_flush ();
+}
+
 /* Handles button releases on the root window via the click_proxy_gdk_window */
 static gint
 click_proxy_button_release (GtkWidget *widget, GdkEventButton *event, gpointer data)
@@ -2657,15 +2674,7 @@ click_proxy_button_release (GtkWidget *widget, GdkEventButton *event, gpointer d
 	if (!click_dragging || event->button != 1)
 		return FALSE;
 
-	draw_rubberband (click_current_x, click_current_y);
-	gdk_pointer_ungrab (event->time);
-	click_dragging = FALSE;
-
-	update_drag_selection (event->x, event->y);
-
-	XUngrabServer (GDK_DISPLAY ());
-	gdk_flush ();
-
+	perform_release (event->time);
 	return TRUE;
 }
 
@@ -2676,6 +2685,17 @@ click_proxy_motion (GtkWidget *widget, GdkEventMotion *event, gpointer data)
 	if (!click_dragging)
 		return FALSE;
 
+	/* There exists the following race condition.  If in the button_press
+	 * handler we manage to grab the server before the window manager can
+	 * proxy the button release to us, then we wil not get the release
+	 * event.  So we have to check the event mask and fake a release by
+	 * hand.
+	 */
+	if (!(event->state & GDK_BUTTON1_MASK)) {
+		perform_release (event->time);
+		return TRUE;
+	}
+
 	draw_rubberband (click_current_x, click_current_y);
 	draw_rubberband (event->x, event->y);
 	update_drag_selection (event->x, event->y);
