@@ -48,10 +48,6 @@ typedef struct {
 
 guint      gtk_dtree_get_type           (void);
 GtkWidget *gtk_dtree_new                (void);
-void       gtk_dtree_select_parent      (GtkDTree *dtree,
-					 char *directory);
-void       gtk_dtree_remove_dir_by_name (GtkDTree *dtree,
-					 char *directory);
 gboolean   gtk_dtree_select_dir         (GtkDTree *dtree,
 					 char *directory);
 char      *gtk_dtree_get_row_path       (GtkDTree *ctree,
@@ -59,4 +55,6 @@ char      *gtk_dtree_get_row_path       (GtkDTree *ctree,
 					 gint column);
 void      gtk_dtree_construct           (GtkDTree *dtree);
 
+gboolean  gtk_dtree_is_ancestor (GtkDTree *dtree, GtkCTreeNode *node, GtkCTreeNode *child);
+
 #endif
