@@ -121,7 +121,7 @@ int altered_magic_flag = 0;
 int altered_nroff_flag = 0;
 /* }}} */
 
-/* "$Id: view.c,v 1.30 1999/04/06 19:00:35 unammx Exp $" */
+/* "$Id: view.c,v 1.34 1999/05/03 18:57:55 unammx Exp $" */
 
 static char hex_char[] = "0123456789ABCDEF";
 
@@ -214,7 +214,7 @@ view_destroy (WView *view)
 }
 
 static int
-get_byte (WView *view, int byte_index)
+get_byte (WView *view, unsigned int byte_index)
 {
     int page   = byte_index / VIEW_PAGE_SIZE + 1;
     int offset = byte_index % VIEW_PAGE_SIZE;
@@ -562,10 +562,23 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
 
     {
         int fd;
-	fd = mc_open(_file, O_RDONLY);
+
+	if ((fd = mc_open(_file, O_RDONLY)) == -1) {
+	    message (1, MSG_ERROR, _(" Can't open \"%s\"\n %s "),
+			_file, unix_error_string (errno));
+	    return -1;
+	}
+	if (mc_fstat (fd, &view->s) == -1) {
+	    message (1, MSG_ERROR, _(" Can't open \"%s\"\n %s "),
+			_file, unix_error_string (errno));
+	    mc_close(fd);
+	    return -1;
+	}
+
 	if (_file[0] && view->viewer_magic_flag && (is_gunzipable (fd, &type)) != 0)
 	    view->filename = g_strconcat (_file, decompress_extension(type), NULL);
-	else view->filename = g_strdup (_file);
+	else 
+	    view->filename = g_strdup (_file);
 	mc_close(fd);
     }
 
@@ -715,11 +728,18 @@ view_status (WView *view, gboolean update_gui)
 	    if (view->growing_buffer)
 		addstr (_("  [grow]"));
 	}
-        if (w > 26)
+        if (w > 26) {
             if (view->hex_mode)
-                view_percent (view, view->edit_cursor - view->first, view->widget.cols - view->have_frame + 1, update_gui);
+                view_percent (view,
+			      view->edit_cursor - view->first,
+			      view->widget.cols - view->have_frame + 1,
+			      update_gui);
             else
-	        view_percent (view, view->start_display - view->first, view->widget.cols - view->have_frame + 1, update_gui);
+	        view_percent (view,
+			      view->start_display - view->first,
+			      view->widget.cols - view->have_frame + 1,
+			      update_gui);
+	}
     }
     attrset (SELECTED_COLOR);
 }
@@ -782,7 +802,7 @@ display (WView *view)
     int col = 0 + frame_shift;
     int row = STATUS_LINES + frame_shift;
     int height, width;
-    long from;
+    unsigned long from;
     int c;
     int boldflag = 0;
     struct hexedit_change_node *curr = view->change_list;
@@ -1078,7 +1098,7 @@ my_define (Dlg_head *h, int idx, char *text,
 static long
 move_forward2 (WView *view, long current, int lines, long upto)
 {
-    long p, q;
+    unsigned long q, p;
     int  line;
     int  col = 0;
 
@@ -1161,7 +1181,7 @@ move_forward2 (WView *view, long current, int lines, long upto)
    of lines up to there and then use move_forward2(p, something, 0), which we
    return */
 static long
-move_backward2 (WView *view, long current, int lines)
+move_backward2 (WView *view, unsigned long current, int lines)
 {
     long p, q, pm;
     int line;
@@ -1375,13 +1395,13 @@ grow_string_buffer (char *text, int *size)
 }
 
 static char *
-get_line_at (WView *view, long *p)
+get_line_at (WView *view, unsigned long *p)
 {
     char *buffer = 0;
     int  buffer_size, usable_size;
     int  ch;
     int  direction;
-    long pos = *p;
+    unsigned long pos = *p;
     long i;
 
     direction = view->direction;
@@ -1597,7 +1617,7 @@ block_search (WView *view, char *buffer, int len)
 {
     int w = view->widget.cols - (view->have_frame * 2);
     char *d = buffer, b;
-    long e;
+    unsigned long e;
 
     /* clear interrupt status */
     got_interrupt ();
@@ -1862,8 +1882,9 @@ void
 goto_line (WView *view)
 {
     char *line, prompt [BUF_SMALL];
-    int i, oldline = 1;
+    int oldline = 1;
     int saved_wrap_mode = view->wrap_mode;
+    unsigned long i;
 
     view->wrap_mode = 0;
     for (i = view->first; i < view->start_display; i++)
@@ -2320,8 +2341,6 @@ view_mode_callback (struct Dlg_head *h, int id, int msg)
 void
 view_adjust_size (Dlg_head *h)
 {
-    int cols;
-    
     WView      *view;
     WButtonBar *bar;
 
